"""The Jupyter HTML Notebook"""

import os

# Packagers: modify the next line if you store the notebook template files
# elsewhere

# Include both notebook/ and notebook/templates/.  This makes it
# possible for users to override a template with a file that inherits from that
# template.
#
# For example, if you want to override a specific block of notebook.html, you
# can create a file called notebook.html that inherits from
# templates/notebook.html, and the latter will resolve correctly to the base
# implementation.
DEFAULT_TEMPLATE_PATH_LIST = [
    os.path.dirname(__file__),
    os.path.join(os.path.dirname(__file__), "templates"),
]

DEFAULT_NOTEBOOK_PORT = 8888

del os

from .nbextensions import install_nbextension
from ._version import version_info, __version__



from selenium.webdriver.common.by import By


def test_prompt_numbers(prefill_notebook):
    notebook = prefill_notebook(['print("a")'])

    def get_prompt():
        return (
            notebook.cells[0].find_element(By.CLASS_NAME, 'input')
            .find_element(By.CLASS_NAME, 'input_prompt')
            .get_attribute('innerHTML').strip()
        )

    def set_prompt(value):
        notebook.set_cell_input_prompt(0, value)

    assert get_prompt() == "<bdi>In</bdi>&nbsp;[&nbsp;]:"

    set_prompt(2)
    assert get_prompt() == "<bdi>In</bdi>&nbsp;[2]:"

    set_prompt(0)
    assert get_prompt() == "<bdi>In</bdi>&nbsp;[0]:"

    set_prompt("'*'")
    assert get_prompt() == "<bdi>In</bdi>&nbsp;[*]:"

    set_prompt("undefined")
    assert get_prompt() == "<bdi>In</bdi>&nbsp;[&nbsp;]:"

    set_prompt("null")
    assert get_prompt() == "<bdi>In</bdi>&nbsp;[&nbsp;]:"



"""Test display isolation.

An object whose metadata contains an "isolated" tag must be isolated
from the rest of the document.
"""

from selenium.webdriver.common.by import By

from .utils import wait_for_tag


def test_display_isolation(notebook):
    import_ln = "from IPython.core.display import HTML, SVG, display, display_svg"
    notebook.edit_cell(index=0, content=import_ln)
    notebook.execute_cell(notebook.current_cell)
    try:
        isolated_html(notebook)
        isolated_svg(notebook)
    finally:
        # Ensure we switch from iframe back to default content even if test fails
        notebook.browser.switch_to.default_content()


def isolated_html(notebook):
    """Test HTML display isolation.

    HTML styling rendered without isolation will affect the whole
    document, whereas styling applied with isolation will affect only
    the local display object.
    """
    red = 'rgb(255, 0, 0)'
    blue = 'rgb(0, 0, 255)'
    test_str = "<div id='test'>Should turn red from non-isolation</div>"
    notebook.add_and_execute_cell(content=f"display(HTML({test_str!r}))")
    non_isolated = (
        f"<style>div{{color:{red};}}</style>"
        f"<div id='non-isolated'>Should be red</div>")
    display_ni = f"display(HTML({non_isolated!r}), metadata={{'isolated':False}})"
    notebook.add_and_execute_cell(content=display_ni)
    isolated = (
        f"<style>div{{color:{blue};}}</style>"
        f"<div id='isolated'>Should be blue</div>")
    display_i = f"display(HTML({isolated!r}), metadata={{'isolated':True}})"
    notebook.add_and_execute_cell(content=display_i)

    iframe = wait_for_tag(notebook.browser, "iframe", single=True)

    # The non-isolated div will be in the body
    non_isolated_div = notebook.body.find_element(By.ID, "non-isolated")
    assert non_isolated_div.value_of_css_property("color") == red

    # The non-isolated styling will have affected the output of other cells
    test_div = notebook.body.find_element(By.ID, "test")
    assert test_div.value_of_css_property("color") == red

    # The isolated div will be in an iframe, only that element will be blue
    notebook.browser.switch_to.frame(iframe)
    isolated_div = notebook.browser.find_element(By.ID, "isolated")
    assert isolated_div.value_of_css_property("color") == blue
    notebook.browser.switch_to.default_content()
    # Clean up the html test cells
    for i in range(1, len(notebook.cells)):
        notebook.delete_cell(1)


def isolated_svg(notebook):
    """Test that multiple isolated SVGs have different scopes.

    Asserts that there no CSS leaks between two isolated SVGs.
    """
    yellow = "rgb(255, 255, 0)"
    black = "rgb(0, 0, 0)"
    svg_1_str = f"""s1 = '''<svg width="1cm" height="1cm" viewBox="0 0 1000 500"><defs><style>rect {{fill:{yellow};}}; </style></defs><rect id="r1" x="200" y="100" width="600" height="300" /></svg>'''"""
    svg_2_str = """s2 = '''<svg width="1cm" height="1cm" viewBox="0 0 1000 500"><rect id="r2" x="200" y="100" width="600" height="300" /></svg>'''"""

    notebook.add_and_execute_cell(content=svg_1_str)
    notebook.add_and_execute_cell(content=svg_2_str)
    notebook.add_and_execute_cell(
        content="display_svg(SVG(s1), metadata=dict(isolated=True))")
    notebook.add_and_execute_cell(
        content="display_svg(SVG(s2), metadata=dict(isolated=True))")
    iframes = wait_for_tag(notebook.browser, "iframe", wait_for_n=2)

    # The first rectangle will be red
    notebook.browser.switch_to.frame(iframes[0])
    isolated_svg_1 = notebook.browser.find_element(By.ID, 'r1')
    assert isolated_svg_1.value_of_css_property("fill") == yellow
    notebook.browser.switch_to.default_content()

    # The second rectangle will be black
    notebook.browser.switch_to.frame(iframes[1])
    isolated_svg_2 = notebook.browser.find_element(By.ID, 'r2')
    assert isolated_svg_2.value_of_css_property("fill") == black

    # Clean up the svg test cells
    for i in range(1, len(notebook.cells)):
        notebook.delete_cell(1)



"""Test the /tree handlers"""
import os
from notebook.utils import url_path_join
from nbformat import write
from nbformat.v4 import new_notebook
from urllib.parse import urlparse

from notebook.tests.launchnotebook import NotebookTestBase

class TreeTest(NotebookTestBase):
    def setUp(self):
        nbdir = self.notebook_dir
        d = os.path.join(nbdir, 'foo')
        os.mkdir(d)

        with open(os.path.join(d, 'bar.ipynb'), 'w', encoding='utf-8') as f:
            nb = new_notebook()
            write(nb, f, version=4)

        with open(os.path.join(d, 'baz.txt'), 'w', encoding='utf-8') as f:
            f.write('flamingo')

        self.base_url()

    def test_redirect(self):
        r = self.request('GET', 'tree/foo/bar.ipynb')
        self.assertEqual(r.url, self.base_url() + 'notebooks/foo/bar.ipynb')

        r = self.request('GET', 'tree/foo/baz.txt', allow_redirects=False)
        self.assertEqual(r.status_code, 302)
        self.assertEqual(r.headers['Location'],
             urlparse(url_path_join(self.base_url(), 'files/foo/baz.txt')).path)



from ..prometheus.metrics import HTTP_REQUEST_DURATION_SECONDS


def prometheus_log_method(handler):
    """
    Tornado log handler for recording RED metrics.

    We record the following metrics:
       Rate - the number of requests, per second, your services are serving.
       Errors - the number of failed requests per second.
       Duration - The amount of time each request takes expressed as a time interval.

    We use a fully qualified name of the handler as a label,
    rather than every url path to reduce cardinality.

    This function should be either the value of or called from a function
    that is the 'log_function' tornado setting. This makes it get called
    at the end of every request, allowing us to record the metrics we need.
    """
    HTTP_REQUEST_DURATION_SECONDS.labels(
        method=handler.request.method,
        handler=f'{handler.__class__.__module__}.{type(handler).__name__}',
        status_code=handler.get_status()
    ).observe(handler.request.request_time())



from notebook.utils import url_path_join
from notebook.tests.launchnotebook import NotebookTestBase

class NbconvertAPI:
    """Wrapper for nbconvert API calls."""
    def __init__(self, request):
        self.request = request

    def _req(self, verb, path, body=None, params=None):
        response = self.request(verb,
                url_path_join('api/nbconvert', path),
                data=body, params=params,
        )
        response.raise_for_status()
        return response

    def list_formats(self):
        return self._req('GET', '')

class APITest(NotebookTestBase):
    def setUp(self):
        self.nbconvert_api = NbconvertAPI(self.request)

    def test_list_formats(self):
        formats = self.nbconvert_api.list_formats().json()
        self.assertIsInstance(formats, dict)
        self.assertIn('python', formats)
        self.assertIn('html', formats)
        self.assertEqual(formats['python']['output_mimetype'], 'text/x-python')